﻿using System;

namespace ScaraBattle.Model
{
    /// <summary>
    /// EventArgs type for the representation of game states
    /// </summary>
    public class GameEventArgs : EventArgs
    {
        public bool GameOver { get; set; }

        public int WhoWon { get; set; }

        public int BlueHealth { get; set; }

        public int BlueGold { get; set; }

        public int RedHealth { get; set; }

        public int RedGold { get; set; }

        public bool SoldTower { get; set; }

        public bool UpgradedTower { get; set; }

        public int SelectedCellX { get; set; }

        public int SelectedCellY { get; set; }

        public String WhyIsNothingHappening { get; set; }

        /// <summary>
        /// Constructor called when the game is over
        /// </summary>
        /// <param name="whoWon">Which player won (1 - blue, 2 - red, 3 - draw)</param>
        public GameEventArgs(int whoWon)
        {
            GameOver = true;
            WhoWon = whoWon;
        }

        /// <summary>
        /// Constructor called when we want to send a message to the user
        /// </summary>
        /// <param name="whyIsNothingHappening">The message to be sent</param>
        public GameEventArgs(string whyIsNothingHappening)
        {
            WhyIsNothingHappening = whyIsNothingHappening;
        }

        /// <summary>
        /// Constructor called when we want to update player stats on the screen
        /// </summary>
        /// <param name="blueHealth">Health of blue player</param>
        /// <param name="blueGold">Gold of blue player</param>
        /// <param name="redHealth">Health of red player</param>
        /// <param name="redGold">Gold of blue player</param>
        public GameEventArgs(int blueHealth, int blueGold, int redHealth, int redGold)
        {
            GameOver = false;
            BlueHealth = blueHealth;
            BlueGold = blueGold;
            RedHealth = redHealth;
            RedGold = redGold;
        }

        /// <summary>
        /// Constructor called when the user selects a cell
        /// </summary>
        /// <param name="selPosY">Y coordinate of selected cell</param>
        /// <param name="selPosX">X coordinate of selected cell</param>
        public GameEventArgs(int selPosY, int selPosX)
        {
            SelectedCellY = selPosY;
            SelectedCellX = selPosX;
        }
    }
}
