﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;

namespace ScaraBattle.Model
{
	/// <summary>
	/// Class implementing perlin noise
	/// </summary>
    public class Perlin
    {

		private Vector[,] _grid;
		private int _n;
		private int _m;

		public Perlin()
        {
			_n = 6;
			_m = 6;
        }

		private double interpolate(double a0, double a1, double w) {
			// You may want clamping by inserting:
			//if (0.0 > w) return a0;
			//if (1.0 < w) return a1;

			return (a1 - a0) * w + a0;
			// Use this cubic interpolation [[Smoothstep]] instead, for a smooth appearance:
			/** return (a1 - a0) * (3.0 - w * 2.0) * w * w + a0;
			*
			* // Use [[Smootherstep]] for an even smoother result with a red derivative equal to zero on boundaries:*/
			// return (a1 - a0) * ((w * (w * 6.0 - 15.0) + 10.0) * w * w * w) + a0;

		}

		public void RandomGrid()
		{

			Random rnd = new Random();
			_grid = new Vector[_n,_m];

			for (int i = 0; i < _n; i++)
			{
				for (int j = 0; j < _m; j++)
				{
					Vector v = new Vector();
					v.X = rnd.Next() % 1000;
					v.Y = rnd.Next() % 1000;
					_grid[i, j] = v;
					_grid[i, j].Normalize();
				}
			}
			
		}

		private double pdistance(int i, int j, double x, double y)
		{
			return Math.Sqrt((i - (float)x) * (i - (float)x) + (j - (float)y) * (j - (float)y));
		}

		private double pdot(int i, int j, double x, double y)
		{
			Vector a = _grid[i,j];
			Vector b = new Vector();
			b.X = x - (double)i;
			b.Y = y - (double)j;
			return a.X*b.X+a.Y*b.Y;
		}

		public double perlinnoise(double x, double y)
		{

			int i = ((int)x);
			
			int j = ((int)y);

			double a = pdot(i, j, x, y);
			double b = pdot(i + 1, j, x, y);
			double c = pdot(i, j + 1, x, y);
			double d = pdot(i + 1, j + 1, x, y);

			double sx = x - (double)i;
			double sy = y - (double)j;
			double x1 = interpolate(a, b, sx);
			double x2 = interpolate(c, d, sx);

			return interpolate(x1, x2, sy);

		}

	}
}
