﻿using System;
using System.Collections.Generic;

namespace ScaraBattle.Persistence
{
    /// <summary>
    /// Enum for the different game states
    /// </summary>
    public enum GameState
    {
        Menu,
        Blue,
        Red,
        Simulation,
        MapEditor
    }

    /// <summary>
    /// Enum for the game object types
    /// </summary>
    public enum SelectedObject
    {
        Nothing,
        Dune,
        Oasis,
        Ruins,
        Castle,
        Barrack,
        SmallTower,
        MediumTower,
        LargeTower,
        SmallUnit,
        LargeUnit
    }

    /// <summary>
    /// Represents the table of the game
    /// </summary>
    public class GameTable
    {
        #region Fields

        private GameObject[,] _cells;

        private int _width;
        private int _height;

        #endregion

        #region Properties

        public GameObject this[int i, int j]
        {
            get { return _cells[i, j]; }
            set { _cells[i, j] = value; }
        }

        public String ColorStandard { get { return "#ffd868"; } }
        public String ColorInjured { get { return "#FF957E";  } }
        public String ColorMarked { get { return "red";  } }
        public int RedLargeUnits { get; set; }
        public int BlueLargeUnits { get; set; }
        public int RedSmallUnits { get; set; }
        public int BlueSmallUnits { get; set; }
        public int RedDeathBonus { get; set; }
        public int BlueDeathBonus { get; set; }
        public int BlueUnitsCurrentNum { get; set; }
        public int RedUnitsCurrentNum { get; set; }
        public int BlueUnitsMaxNum { get; set; }
        public int RedUnitsMaxNum { get; set; }
        public List<List<List<Unit>>> BlueUnitsCurrentPositions { get; set; }
        public List<List<List<Unit>>> RedUnitsCurrentPositions { get; set; }
        public Castle BlueCastle { get; set; }
        public (int Y, int X)[] BlueBarracks { get; set; }
        public List<(int Y, int X)> BlueTowerPositions { get; set; }
        public int BlueGold { get; set; }
        public int BlueHealth { get { return BlueCastle.Health; } set { BlueCastle.Health = value; } }
        public List<Unit> BlueUnits { get; set; }

        public Castle RedCastle { get; set; }
        public (int, int)[] RedBarracks { get; set; }
        public List<(int Y, int X)> RedTowerPositions { get; set; }
        public int RedGold { get; set; }
        public int RedHealth { get { return RedCastle.Health; } set { RedCastle.Health = value; } }

        public List<Unit> RedUnits { get; set; }

        public int Width { get { return _width; } }

        public int Height { get { return _height; } }

        public GameState State { get; set; }

        #endregion

        #region Constructors

        /// <summary>
        /// Constructs a table with the given size
        /// </summary>
        /// <param name="height">Height of the table</param>
        /// <param name="width">Width of the table</param>
        public GameTable(int height, int width, GameState state, int startGold = 100)
        {
            _height = height;
            _width = width;
            _cells = new GameObject[height, width];
            State = state;
            BlueGold = startGold;
            RedGold = startGold;
            BlueTowerPositions = new List<(int, int)>();
            RedTowerPositions = new List<(int, int)>();
            BlueCastle = new Castle();
            RedCastle = new Castle();
            BlueUnitsCurrentPositions = new List<List<List<Unit>>>();
            RedUnitsCurrentPositions = new List<List<List<Unit>>>();
            BlueUnitsCurrentNum = 0;
            RedUnitsCurrentNum = 0;
            BlueUnitsMaxNum = 0;
            RedUnitsMaxNum = 0;
            BlueDeathBonus = 0;
            RedDeathBonus = 0;
            RedLargeUnits = 0;
            BlueLargeUnits = 0;
            RedSmallUnits = 0;
            BlueSmallUnits = 0;
            BlueUnits = new List<Unit>();
            RedUnits = new List<Unit>();
        }

        #endregion
    }
}
